#!/bin/bash

sum16() {
   SUM=0
   for (( i=0; i<${#1}; i=i+2 ))
   do
      SUM=$(( $SUM + 0x${1:$i:2} ))
   done
   echo $(( $SUM & 0xFFFF ))
}

AUTHOR="Janne Hellas"
VERSION="1.0"
LICENSE="Public Domain or equivalent."
LINFO="This software is provided \"as-is\" without any warranties or liabilities."

# Test environment
BC=`which bc`
if [[ "${BC}" == "" ]]; then
   echo "$0: Environmental failure. Install bc." 1>&2
   exit 1
fi

# Handle non-option arguments
if [[ $# -eq 0 ]]; then
   echo "FF6 GBA save file checksum updater v$VERSION"
   echo "Updates the known save file checksums."
   echo ""
   echo "Usage: ./ff6gba_checksumupdater.sh [save file]"
   echo ""
   echo "License: $LICENSE"
   echo "$LINFO"
   exit 0
fi

if [[ $# -ne 1 ]]; then
   echo "$0: A single input file is required." 1>&2
   exit 2
fi

if [ ! -f "$1" ]; then
   echo "$0: Input file doesn't exist." 1>&2
   exit 3
fi

SLOT=1
OUTFILE="$1"

SCRIPTDIR=$(dirname "${BASH_SOURCE}")

# Detect format
ADV=`${SCRIPTDIR}/ff6detect.sh "$1"`
if [ "${ADV}" != "ff6advance" ]; then
    echo "$0: Input file is not detected as a save." 1>&2
    exit 2
fi

# Data1 checksum
DATAHEX=`head -c 2558 "$OUTFILE" | xxd -p -u | tr -d " \n"`
DATA1CHECKSUM=`sum16 $DATAHEX`
DATA1CHECKSUM=$(echo "obase=16; ibase=10; $DATA1CHECKSUM" | bc)
DATA1CHECKSUM=$(printf '%04X\n' "0x$DATA1CHECKSUM")

echo "Writing data1 checksum: $DATA1CHECKSUM"

printf "$(printf '\\x%02X' 0x${DATA1CHECKSUM:2:2})" | dd of="$OUTFILE" bs=1 seek=2558 count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${DATA1CHECKSUM:0:2})" | dd of="$OUTFILE" bs=1 seek=2559 count=1 conv=notrunc status=none

# Data1 extension checksum
DATAHEX=`tail -c +8193 "$OUTFILE" | head -c 1022 | xxd -p -u | tr -d " \n"`
SUM16=`sum16 $DATAHEX`
EXT1CHECKSUM=$(echo "obase=16; ibase=10; $(( $SUM16 + 1 ))" | bc)
EXT1CHECKSUM=$(printf '%04X\n' "0x$EXT1CHECKSUM")

echo "Writing data1 extension checksum: $EXT1CHECKSUM"

printf "$(printf '\\x%02X' 0x${EXT1CHECKSUM:2:2})" | dd of="$OUTFILE" bs=1 seek=$(( 8192 + 1022 )) count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${EXT1CHECKSUM:0:2})" | dd of="$OUTFILE" bs=1 seek=$(( 8192 + 1023 )) count=1 conv=notrunc status=none

# Bestiary checksum
DATAHEX=`tail -c +11265 "$OUTFILE" | head -c 1022 | xxd -p -u | tr -d " \n"`
SUM16=`sum16 $DATAHEX`
BECHECKSUM=$(echo "obase=16; ibase=10; $(( $SUM16 + 1 ))" | bc)
BECHECKSUM=$(printf '%04X\n' "0x$BECHECKSUM")

echo "Writing bestiary checksum: $BECHECKSUM"

printf "$(printf '\\x%02X' 0x${BECHECKSUM:2:2})" | dd of="$OUTFILE" bs=1 seek=$(( 11264 + 1022 )) count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${BECHECKSUM:0:2})" | dd of="$OUTFILE" bs=1 seek=$(( 11264 + 1023 )) count=1 conv=notrunc status=none

echo "Script run completed."
