#!/bin/bash

sum16() {
   SUM=0
   for (( i=0; i<${#1}; i=i+2 ))
   do
      SUM=$(( $SUM + 0x${1:$i:2} ))
   done
   echo $(( $SUM & 0xFFFF ))
}

crc32() {
   CRC32=`cat "$1" | gzip -c | tail -c 8 | head -c 4 | xxd -p -u`
   CRC32=$(printf '%08X\n' "0x$CRC32")
   echo "${CRC32:6:2}${CRC32:4:2}${CRC32:2:2}${CRC32:0:2}"
}

AUTHOR="Janne Hellas"
VERSION="1.0"
LICENSE="Public Domain or equivalent."
LINFO="This software is provided \"as-is\" without any warranties or liabilities."

# Test environment
BC=`which bc`
if [[ "${BC}" == "" ]]; then
   echo "$0: Environmental failure. Install bc." 1>&2
   exit 1
fi

# Handle non-option arguments
if [[ $# -eq 0 ]]; then
   echo "FF6 Mobile/PC save file checksum updater v$VERSION"
   echo "Updates the known save file checksums."
   echo ""
   echo "Usage: ./ff6mob_checksumupdater.sh [save file]"
   echo ""
   echo "License: $LICENSE"
   echo "$LINFO"
   exit 0
fi

if [[ $# -ne 1 ]]; then
   echo "$0: A single input file is required." 1>&2
   exit 2
fi

if [ ! -f "$1" ]; then
   echo "$0: Input file not found." 1>&2
   exit 3
fi

SLOT=1
OUTFILE="$1"
TEMPFILE=/tmp/ff6mobwriter$$

SCRIPTDIR=$(dirname "${BASH_SOURCE}")

#Detect ff6mobile
FF6MOBILE=`"${SCRIPTDIR}/ff6detect.sh" "$1"`
if [ "${FF6MOBILE}" != "ff6mobile" ]; then
   echo "$0: Input file format detection failure. Check that the save file is from ff6mobile." 1>&2
   exit 4
fi

# Data1 checksum
DATAHEX=`tail -c +129 "$OUTFILE" | head -c 2558 | xxd -p -u | tr -d " \n"`
DATA1CHECKSUM=$(echo "obase=16; ibase=10; `sum16 $DATAHEX`" | bc)
DATA1CHECKSUM=$(printf '%04X\n' "0x$DATA1CHECKSUM")

echo "Writing data1 checksum: $DATA1CHECKSUM"

printf "$(printf '\\x%02X' 0x${DATA1CHECKSUM:2:2})" | dd of="$OUTFILE" bs=1 seek=$(( 2558 + 128 )) count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${DATA1CHECKSUM:0:2})" | dd of="$OUTFILE" bs=1 seek=$(( 2559 + 128 )) count=1 conv=notrunc status=none

# Data1 extension checksum
DATAHEX=`tail -c +2689 "$OUTFILE" | head -c 1022 | xxd -p -u | tr -d " \n"`
SUM16=`sum16 $DATAHEX`
EXT1CHECKSUM=$(echo "obase=16; ibase=10; $(( SUM16 + 1 ))" | bc)
EXT1CHECKSUM=$(printf '%04X\n' "0x$EXT1CHECKSUM")

echo "Writing data1 extension checksum: $EXT1CHECKSUM"

printf "$(printf '\\x%02X' 0x${EXT1CHECKSUM:2:2})" | dd of="$OUTFILE" bs=1 seek=$(( 2688 + 1022 )) count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${EXT1CHECKSUM:0:2})" | dd of="$OUTFILE" bs=1 seek=$(( 2688 + 1023 )) count=1 conv=notrunc status=none

# Bestiary checksum
DATAHEX=`tail -c +3713 "$OUTFILE" | head -c 1022 | xxd -p -u | tr -d " \n"`
SUM16=`sum16 $DATAHEX`
BECHECKSUM=$(echo "obase=16; ibase=10; $(( SUM16 + 1 ))" | bc)
BECHECKSUM=$(printf '%04X\n' "0x$BECHECKSUM")

echo "Writing bestiary checksum: $BECHECKSUM"

printf "$(printf '\\x%02X' 0x${BECHECKSUM:2:2})" | dd of="$OUTFILE" bs=1 seek=$(( 3712 + 1022 )) count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${BECHECKSUM:0:2})" | dd of="$OUTFILE" bs=1 seek=$(( 3712 + 1023 )) count=1 conv=notrunc status=none


# Slot checksum
tail -c +129 "$OUTFILE" | head -c 20992 > "$TEMPFILE"
SLOTCHECKSUM=`crc32 "$TEMPFILE"`

echo "Writing slot checksum: $SLOTCHECKSUM"

printf "$(printf '\\x%02X' 0x${SLOTCHECKSUM:6:2})" | dd of="$OUTFILE" bs=1 seek=112 count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${SLOTCHECKSUM:4:2})" | dd of="$OUTFILE" bs=1 seek=$(( 112 + 1 )) count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${SLOTCHECKSUM:2:2})" | dd of="$OUTFILE" bs=1 seek=$(( 112 + 2 )) count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${SLOTCHECKSUM:0:2})" | dd of="$OUTFILE" bs=1 seek=$(( 112 + 3 )) count=1 conv=notrunc status=none

# File checksum
# Set zeros to file checksum field
printf "$(printf '\\x%02X' 0x0)" | dd of="$OUTFILE" bs=1 seek=96 count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x0)" | dd of="$OUTFILE" bs=1 seek=97 count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x0)" | dd of="$OUTFILE" bs=1 seek=98 count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x0)" | dd of="$OUTFILE" bs=1 seek=99 count=1 conv=notrunc status=none

tail -c +17 "$OUTFILE" > "$TEMPFILE"

FILECHECKSUM=`crc32 "$TEMPFILE"`

echo "Writing file checksum: $FILECHECKSUM"

printf "$(printf '\\x%02X' 0x${FILECHECKSUM:6:2})" | dd of="$OUTFILE" bs=1 seek=96 count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${FILECHECKSUM:4:2})" | dd of="$OUTFILE" bs=1 seek=$(( 96 + 1 )) count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${FILECHECKSUM:2:2})" | dd of="$OUTFILE" bs=1 seek=$(( 96 + 2 )) count=1 conv=notrunc status=none
printf "$(printf '\\x%02X' 0x${FILECHECKSUM:0:2})" | dd of="$OUTFILE" bs=1 seek=$(( 96 + 3 )) count=1 conv=notrunc status=none

# Clean up
if [ -f "$TEMPFILE" ]; then
   rm "$TEMPFILE"
fi

echo "Script run completed."
